@@ -10,12 +10,18 @@ use HTTP::Request;
 use XML::Simple;
 use Data::Dumper;
 use Time::Local;
+use base qw(Weather::Com::Object Exporter);
 
-use base qw(Exporter);
-our @EXPORT_OK =
-  qw ( celsius2fahrenheit fahrenheit2celsius convert_winddirection);
+#--------------------------------------------------------------------
+# Define some globals
+#--------------------------------------------------------------------
+our @EXPORT_OK = qw( 
+	celsius2fahrenheit 
+	fahrenheit2celsius 
+	convert_winddirection
+);
 
-our $VERSION = sprintf "%d.%03d", q$Revision: 1.4 $ =~ /(\d+)/g;
+our $VERSION = sprintf "%d.%03d", q$Revision: 1.1 $ =~ /(\d+)/g;
 
 my $CITY_SEARCH_URI    = "http://xoap.weather.com/search/search?where=";
 my $WEATHER_SEARCH_URI = "http://xoap.weather.com/weather/local/";
@@ -66,7 +72,17 @@ my %winddir = (
 sub new {
 	my $proto = shift;
 	my $class = ref($proto) || $proto;
-	my $self  = {};
+	my %parameters;
+	
+	# parameters provided by new method
+	if ( ref( $_[0] ) eq "HASH" ) {
+		%parameters = %{ $_[0] };
+	} else {
+		%parameters = @_;
+	}
+
+	# get SUPER instance
+	my $self = $class->SUPER::new( \%parameters );
 
 	# some general attributes
 	$self->{PROXY}   = "none";
@@ -81,23 +97,12 @@ sub new {
 	$self->{UNITS}    = 'm';    # could be 'm' for metric or 's' for us standard
 	$self->{CC}       = 0;      # show current conditions true/false
 	$self->{FORECAST} = 0;      # multi day forecast 0 = no, 1..10 days
-	$self->{LINKS}    = 0;
-
-	# parameters provided by new method
-	my %parameters = ();
-	if ( ref( $_[0] ) eq "HASH" ) {
-		%parameters = %{ $_[0] };
-	} else {
-		%parameters = @_;
-	}
 
+	# save params for further use
 	$self->{ARGS} = \%parameters;
 
-	# bless out object
-	bless( $self, $class );
-
 	# do some initialization with validity checking
-	$self = $self->_init( \%parameters );
+	$self = $self->_init();
 
 	# and some debugging output
 	$self->_debug( "Returning object: " . Dumper($self) );
@@ -129,7 +134,7 @@ sub _init {
 
 	# set timeout if it is a possitive integer or 0
 	if ( $params->{timeout} ) {
-		unless (    ( $params->{timout} =~ /^\d+$/ )
+		unless (    ( $params->{timeout} =~ /^\d+$/ )
 				 && ( $params->{timeout} > -1 ) )
 		{
 			die ref($self)
@@ -163,7 +168,12 @@ sub _init {
 	$self->{LICENSE_KEY} = $params->{license} if ( $params->{license} );
 	$self->{DEBUG}       = $params->{debug}   if ( $params->{debug} );
 	$self->{CC}          = $params->{current} if ( $params->{current} );
-	$self->{LINKS}       = $params->{links}   if ( $params->{links} );
+
+	if ( $params->{lang} ) {
+		$self->{LANGUAGE} = $params->{lang};
+	} else {
+		$self->{LANGUAGE} = 'en_US';
+	}
 
 	return $self;
 }
@@ -183,7 +193,7 @@ sub search {
 	# build up HTTP GET request
 	$place = uri_escape($place);
 	my $searchlocation = $CITY_SEARCH_URI . $place;
-	$searchlocation .= '&prod=xoap';
+	# $searchlocation .= '&prod=xoap';
 	if ( $self->{PARTNER_ID} && $self->{LICENSE_KEY} ) {
 		$searchlocation .= '&par=' . $self->{PARTNER_ID};
 		$searchlocation .= '&key=' . $self->{LICENSE_KEY};
@@ -257,9 +267,9 @@ sub get_weather {
 	if ( $self->{FORECAST} ) {
 		$searchlocation .= '&dayf=' . $self->{FORECAST};
 	}
-	if ( $self->{LINKS} ) {
-		$searchlocation .= '&link=xoap';
-	}
+
+	# add 'link' parameter in any case
+	$searchlocation .= '&link=xoap';
 
 	# get weather data
 	$self->_debug($searchlocation);
@@ -281,6 +291,8 @@ sub get_weather {
 		die ref($self), ": ERROR ", $simpleHash->{err}->{content}, "\n";
 	}
 
+	$self->_debug(Dumper($simpleHash));
+
 	return $simpleHash;
 }
 
@@ -296,10 +308,10 @@ sub _getWebPage {
 	my $request = HTTP::Request->new( "GET" => $path );
 
 	$ua->proxy( 'http', $self->{PROXY} )
-	  if ( lc( $self->{PROXY} ) ne "none" );
+	  if ( defined $self->{PROXY} && lc( $self->{PROXY} ) ne "none" );
 	$request->proxy_authorization_basic( $self->{PROXY_USER},
 										 $self->{PROXY_PASS} )
-	  if ( lc( $self->{PROXY_USER} ) ne "none" );
+	  if ( defined $self->{PROXY_USER} && lc( $self->{PROXY_USER} ) ne "none" );
 	$ua->timeout( $self->{TIMEOUT} );
 
 	# print some debugging info on the user agent object
@@ -448,7 +460,6 @@ Weather::Com::Base - Perl extension for getting weather information from weather
   my %params = (
 		'current'    => 1,
 		'forecast'   => 3,
-		'links'      => 1,
 		'units'      => 's',
 		'proxy'      => 'http://proxy.sonstwo.de',
 		'timeout'    => 250,
@@ -557,11 +568,6 @@ set to any other value, this is interpreted as 0!
 
 Defaults to 0 (false).
 
-=item links => 0 | 1
-
-This parameter specifies whether to load some http links from I<weather.com>
-also or not.
-
 =item units => s | m
 
 This parameter defines whether to fetch information in metric (m) or 
@@ -1035,7 +1041,7 @@ Thomas Schnuecker, E<lt>thomas@schnuecker.deE<gt>
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright (C) 2004 by Thomas Schnuecker
+Copyright (C) 2004-2009 by Thomas Schnuecker
 
 This library is free software; you can redistribute it and/or modify
 it under the same terms as Perl itself.
